@@ -1,4 +1,4 @@
-from fireworks import LaunchPad, explicit_serialize, FireTaskBase, FWAction, Firework, Workflow
+from fireworks import LaunchPad, explicit_serialize, FiretaskBase, FWAction, Firework, Workflow
 from fireworks.core.rocket_launcher import rapidfire
 
 __author__ = 'Anubhav Jain <ajain@lbl.gov>'
@@ -7,8 +7,9 @@ __author__ = 'Anubhav Jain <ajain@lbl.gov>'
 A workflow where 2 parent Fireworks (A and B) merge into a common child C.
 """
 
+
 @explicit_serialize
-class TaskA(FireTaskBase):
+class TaskA(FiretaskBase):
 
     def run_task(self, fw_spec):
         print("This is task A")
@@ -16,20 +17,22 @@ class TaskA(FireTaskBase):
 
 
 @explicit_serialize
-class TaskB(FireTaskBase):
+class TaskB(FiretaskBase):
 
     def run_task(self, fw_spec):
         print("This is task B")
         return FWAction(update_spec={"param_B": 20})
 
+
 @explicit_serialize
-class TaskC(FireTaskBase):
+class TaskC(FiretaskBase):
 
     def run_task(self, fw_spec):
         print("This is task C.")
         print("Task A gave me: {}".format(fw_spec["param_A"]))
         print("Task B gave me: {}".format(fw_spec["param_B"]))
 
+
 if __name__ == "__main__":
     # set up the LaunchPad and reset it
     launchpad = LaunchPad()
@@ -44,4 +47,4 @@ if __name__ == "__main__":
 
     # store workflow and launch it locally
     launchpad.add_wf(workflow)
-    rapidfire(launchpad)
\ No newline at end of file
+    rapidfire(launchpad)
