@@ -5,7 +5,7 @@ from __future__ import unicode_literals, absolute_import
 import multiprocessing
 
 import gunicorn.app.base
-from gunicorn.six import iteritems
+from six import iteritems
 from fireworks.flask_site.app import app as handler_app
 
 
@@ -31,6 +31,7 @@ class StandaloneApplication(gunicorn.app.base.BaseApplication):
     def load(self):
         return self.application
 
+
 if __name__ == '__main__':
     options = {
         'bind': '%s:%s' % ('127.0.0.1', '8080'),
