@@ -3,18 +3,13 @@
 from __future__ import unicode_literals
 
 """
-A set of global constants for FireWorks (Python code as a config file)
+A set of global constants for FireWorks (Python code as a config file).
 """
 
 import os
 from monty.serialization import loadfn, dumpfn
 from monty.design_patterns import singleton
 
-try:
-    from yaml import CSafeDumper as Dumper
-except ImportError:
-    from yaml import SafeDumper as Dumper
-
 __author__ = 'Anubhav Jain'
 __copyright__ = 'Copyright 2012, The Materials Project'
 __version__ = '0.1'
@@ -25,16 +20,14 @@ __date__ = 'Dec 12, 2012'
 NEGATIVE_FWID_CTR = 0
 
 # this is where load_object() looks for serialized objects
-
 USER_PACKAGES = ['fireworks.user_objects', 'fireworks.utilities.tests',
                  'fw_tutorials', 'fireworks.features']
 
+# if you update a _fw_name, you can use this to record the change and maintain deserialization
 FW_NAME_UPDATES = {'Transfer Task': 'FileTransferTask',
                    'Script Task': 'ScriptTask',
                    'Template Writer Task': 'TemplateWriterTask',
                    'Dupe Finder Exact': 'DupeFinderExact'}
-# if you update a _fw_name, you can use this to record the change and
-# maintain deserialization
 
 YAML_STYLE = False  # controls whether YAML documents will be nested as braces or blocks (False = blocks)
 
@@ -70,6 +63,8 @@ QUEUEADAPTER_LOC = None  # where to find the my_qadapter.yaml file
 
 CONFIG_FILE_DIR = '.'  # directory containing config files (if not individually set)
 
+ROCKET_STREAM_LOGLEVEL = "INFO"  # the streaming log level of the rocket.launcher logger
+
 QSTAT_FREQUENCY = 50  # set this higher to avoid qstats, lower to alwas
 
 ALWAYS_CREATE_NEW_BLOCK = False  # always create new block on queue launcher call
@@ -98,8 +93,15 @@ WEBSERVER_HOST = "127.0.0.1"  # default host on which the Flask web server runs
 
 WEBSERVER_PORT = 5000  # default port on which the Flask web server runs
 
-MONGO_SOCKET_TIMEOUT_MS = 5 * 60 * 1000  # value of socketTimeoutMS when connection to mongoDB.  See pymongo official
-                                         # documentation http://api.mongodb.org/python/current/api/pymongo/mongo_client.html
+WEBSERVER_PERFWARNINGS = False  # enable performance-related warnings
+
+# value of socketTimeoutMS when connection to mongoDB.  See pymongo official
+# documentation http://api.mongodb.org/python/current/api/pymongo/mongo_client.html
+MONGO_SOCKET_TIMEOUT_MS = 5 * 60 * 1000
+
+# name of the collection that will be used to store information in case the size of
+# a dynamically generated document exceeds the 16MB limit. Functionality disabled if None.
+GRIDFS_FALLBACK_COLLECTION = "fw_gridfs"
 
 
 def override_user_settings():
@@ -115,7 +117,7 @@ def override_user_settings():
         if fp not in config_paths and os.path.exists(fp):
             config_paths.append(fp)
 
-    if "FW_CONFIG_FILE" in os.environ and os.environ["FW_CONFIG_FILE"] not in\
+    if "FW_CONFIG_FILE" in os.environ and os.environ["FW_CONFIG_FILE"] not in \
             config_paths:
         config_paths.append(os.environ["FW_CONFIG_FILE"])
 
@@ -134,9 +136,7 @@ def override_user_settings():
             elif key == 'ECHO_TEST':
                 print(v)
             elif key not in globals():
-                raise ValueError(
-                    'Invalid FW_config file has unknown parameter: {}'.format(
-                        key))
+                raise ValueError('Invalid FW_config file has unknown parameter: {}'.format(key))
             else:
                 globals()[key] = v
 
@@ -172,15 +172,14 @@ def config_to_dict():
 
 
 def write_config(path=None):
-    path = os.path.join(os.path.expanduser('~'), ".fireworks",
-                        'FW_config.yaml') if path is None else path
-    dumpfn(config_to_dict(), path, Dumper=Dumper)
+    path = os.path.join(os.path.expanduser('~'), ".fireworks", 'FW_config.yaml') if path is None else path
+    dumpfn(config_to_dict(), path)
 
 
 @singleton
 class FWData(object):
     """
-    This class stores data that a FireTask might want to access, e.g. to see the runtime params
+    This class stores data that a Firetask might want to access, e.g. to see the runtime params
     """
 
     def __init__(self):
