@@ -5,6 +5,7 @@ from __future__ import unicode_literals
 """
 This module contains contracts for defining adapters to various queueing systems, e.g. PBS/SLURM/SGE.
 """
+
 import os
 import shlex
 import string
@@ -13,6 +14,9 @@ import threading
 import traceback
 import abc
 import collections
+import six
+import warnings
+
 from fireworks.utilities.fw_serializers import FWSerializable, serialize_fw
 from fireworks.utilities.fw_utilities import get_fw_logger
 
@@ -39,19 +43,25 @@ class Command(object):
 
     def __init__(self, command):
         """
-        initialize the object
-        :param command: command to run
+        initialize the object.
+
+        Args:
+            command: command to run
         """
-        if isinstance(command, basestring):
+        if isinstance(command, six.string_types):
             command = shlex.split(command)
         self.command = command
 
     def run(self, timeout=None, **kwargs):
         """
-        Run the command
-        :param timeout: (float) timeout
-        :param kwargs:
-        :return: (status, output, error)
+        Run the command.
+
+        Args:
+            timeout (float): timeout
+            kwargs (dict)
+
+        Returns:
+            (status, output, error)
         """
         def target(**kwargs):
             try:
@@ -64,7 +74,7 @@ class Command(object):
                     self.output = self.output.decode("utf-8")
                 if isinstance(self.error, bytes):
                     self.error = self.error.decode("utf-8")
-            except:
+            except Exception:
                 self.error = traceback.format_exc()
                 self.status = -1
         # default stdout and stderr
@@ -84,16 +94,15 @@ class Command(object):
 
 class QueueAdapterBase(collections.defaultdict, FWSerializable):
     """
-    The QueueAdapter is responsible for all interactions with a specific \
-    queue management system. This includes handling all details of queue \
-    script format as well as queue submission and management.
+    The QueueAdapter is responsible for all interactions with a specific queue management system.
+    This includes handling all details of queue script format as well as queue submission and
+     management.
 
-    A user should extend this class with implementations that work on \
-    specific queue systems. Examples and implementations are in: \
-    fireworks/user_objects/queue_adapters.
+    A user should extend this class with implementations that work on specific queue systems.
+    Examples and implementations are in: fireworks/user_objects/queue_adapters.
 
-    Documentation on implementing queue adapters can be found on FireWorks \
-    home page, http://pythonhosted.org/FireWorks
+    Documentation on implementing queue adapters can be found on FireWorks home page,
+    https://materialsproject.github.io/fireworks
     """
 
     _fw_name = 'QueueAdapterBase'
@@ -104,19 +113,35 @@ class QueueAdapterBase(collections.defaultdict, FWSerializable):
 
     def get_script_str(self, launch_dir):
         """
-        returns a (multi-line) String representing the queue script, e.g. PBS script. \
+        returns a (multi-line) String representing the queue script, e.g. PBS script.
         Uses the template_file along with internal parameters to create the script.
 
-        :param launch_dir: (str) The directory the job will be launched in
-        :return: (str) the queue script
+        Args:
+            launch_dir (str): The directory the job will be launched in
+
+        Returns:
+            (str) the queue script
         """
         with open(self.template_file) as f:
-            a = QScriptTemplate(f.read())
+            template = f.read()
+            a = QScriptTemplate(template)
+
+            # get keys defined by template
+            template_keys = [i[1] for i in string.Formatter().parse(template)]
 
             # set substitution dict for replacements into the template
             subs_dict = {k: v for k, v in self.items()
                          if v is not None}  # clean null values
 
+            # warn user if they specify a key not present in template
+            for subs_key in subs_dict.keys():
+                if subs_key not in template_keys and not \
+                        subs_key.startswith("_") and not subs_key == "logdir":
+                    warnings.warn('Key {} has been specified in qadapter '
+                                  'but it is not present in template, please '
+                                  'check template ({}) for supported keys.'
+                                  .format(subs_key, self.template_file))
+
             for k, v in self.defaults.items():
                 subs_dict.setdefault(k, v)
 
@@ -136,20 +161,26 @@ class QueueAdapterBase(collections.defaultdict, FWSerializable):
     @abc.abstractmethod
     def submit_to_queue(self, script_file):
         """
-        submits the job to the queue and returns the job id
+        Submits the job to the queue and returns the job id.
 
-        :param script_file: (str) name of the script file to use (String)
-        :return: (int) job_id
+        Args:
+            script_file: (str) name of the script file to use (String)
+
+        Returns:
+            (int) job_id
         """
         pass
 
     @abc.abstractmethod
     def get_njobs_in_queue(self, username=None):
         """
-        returns the number of jobs currently in the queue for the user
+        Returns the number of jobs currently in the queue for the user.
 
-        :param username: (str) the username of the jobs to count (default is to autodetect)
-        :return: (int) number of jobs in the queue
+        Args:
+            username (str): the username of the jobs to count (default is to autodetect)
+
+        Returns:
+            (int) number of jobs in the queue
         """
         pass
 
