@@ -11,12 +11,11 @@ import glob
 import unittest
 import time
 from fireworks import explicit_serialize, FWAction
-from fireworks.core.firework import Firework, Workflow, FireTaskBase
+from fireworks.core.firework import Firework, Workflow, FiretaskBase
 from fireworks.core.fworker import FWorker
 from fireworks.core.launchpad import LaunchPad, WFLock
 from fireworks.core.rocket_launcher import launch_rocket, rapidfire
 from fireworks.features.background_task import BackgroundTask
-from fireworks.fw_config import WFLOCK_EXPIRATION_KILL
 from fireworks.queue.queue_launcher import setup_offline_job
 from fireworks.user_objects.dupefinders.dupefinder_exact import DupeFinderExact
 from fireworks.user_objects.firetasks.fileio_tasks import FileTransferTask, FileWriteTask
@@ -26,7 +25,6 @@ from fw_tutorials.dynamic_wf.fibadd_task import FibonacciAdderTask
 from fw_tutorials.firetask.addition_task import AdditionTask
 from fireworks.tests.tasks import DummyFWEnvTask, DummyJobPassTask, DummyLPTask
 from fireworks.features.stats import FWStats
-import six
 
 __author__ = 'Anubhav Jain'
 __copyright__ = 'Copyright 2013, The Materials Project'
@@ -49,7 +47,7 @@ def throw_error(msg):
     raise ValueError(msg)
 
 @explicit_serialize
-class MultipleDetourTask(FireTaskBase):
+class MultipleDetourTask(FiretaskBase):
 
     def run_task(self, fw_spec):
         print('Running the Multiple Detour Task')
@@ -59,14 +57,14 @@ class MultipleDetourTask(FireTaskBase):
         return FWAction(detours=[dt1, dt2, dt3])
 
 @explicit_serialize
-class UpdateSpecTask(FireTaskBase):
+class UpdateSpecTask(FiretaskBase):
     def run_task(self, fw_spec):
         print('Running the Update Spec Task')
         dt1 = Firework(ScriptTask.from_str('echo "this is dummy job 1"'))
         return FWAction(update_spec={"dummy1": 1}, additions=[dt1])
 
 @explicit_serialize
-class ModSpecTask(FireTaskBase):
+class ModSpecTask(FiretaskBase):
     def run_task(self, fw_spec):
         print('Running the Mod Spec Task')
         return FWAction(mod_spec=[{"_push": {"dummy2": True}}])
@@ -81,7 +79,7 @@ class MongoTests(unittest.TestCase):
         try:
             cls.lp = LaunchPad(name=TESTDB_NAME, strm_lvl='ERROR')
             cls.lp.reset(password=None, require_password=False)
-        except:
+        except Exception:
             raise unittest.SkipTest('MongoDB is not running in localhost:27017! Skipping tests.')
 
     @classmethod
@@ -89,7 +87,8 @@ class MongoTests(unittest.TestCase):
         if cls.lp:
             cls.lp.connection.drop_database(TESTDB_NAME)
 
-    def _teardown(self, dests):
+    @staticmethod
+    def _teardown(dests):
         for f in dests:
             if os.path.exists(f):
                 os.remove(f)
@@ -357,6 +356,32 @@ class MongoTests(unittest.TestCase):
 
         self.assertEqual(len(modified_spec['_job_info']), 2)
 
+    def test_files_in_out(self):
+        # create the Workflow that passes files_in and files_out
+        fw1 = Firework(
+            [ScriptTask.from_str('echo "This is the first FireWork" > test1')],
+            spec={"_files_out": {"fwtest1": "test1"}}, fw_id=1)
+        fw2 = Firework([ScriptTask.from_str('gzip hello')], fw_id=2,
+                       parents=[fw1],
+                       spec={"_files_in": {"fwtest1": "hello"},
+                             "_files_out": {"fw2": "hello.gz"}})
+        fw3 = Firework([ScriptTask.from_str('cat fwtest.2')], fw_id=3,
+                       parents=[fw2],
+                       spec={"_files_in": {"fw2": "fwtest.2"}})
+        wf = Workflow([fw1, fw2, fw3],
+                      {fw1: [fw2], fw2: [fw3]})
+
+        # store workflow and launch it locally
+        self.lp.add_wf(wf)
+        launch_rocket(self.lp, self.fworker)
+        self.assertTrue(os.path.exists("test1"))
+        launch_rocket(self.lp, self.fworker)
+        self.assertTrue(os.path.exists("hello.gz"))
+        launch_rocket(self.lp, self.fworker)
+        self.assertTrue(os.path.exists("fwtest.2"))
+        for f in ["test1", "hello.gz", "fwtest.2"]:
+            os.remove(f)
+
     def test_preserve_fworker(self):
         fw1 = Firework([ScriptTask.from_str('echo "Testing preserve FWorker"')], spec={"_preserve_fworker": True}, fw_id=1)
         fw2 = Firework([ScriptTask.from_str('echo "Testing preserve FWorker pt 2"')], spec={"target": 1}, parents=[fw1], fw_id=2)
@@ -399,6 +424,38 @@ class MongoTests(unittest.TestCase):
         self.assertEqual(self.lp.get_fw_by_id(1).tasks[0]['script'][0], 'echo "Task 1"')
         self.assertEqual(self.lp.get_fw_by_id(2).tasks[0]['script'][0], 'echo "Task 2"')
 
+    def test_category(self):
+        task1 = ScriptTask.from_str('echo "Task 1"')
+        task2 = ScriptTask.from_str('echo "Task 2"')
+
+        spec = {'_category': 'dummy_category'}
+
+        fw1 = Firework(task1, fw_id=1, name='Task 1', spec=spec)
+        fw2 = Firework(task2, fw_id=2, name='Task 2', spec=spec)
+
+        self.lp.add_wf(Workflow([fw1, fw2]))
+
+        self.assertTrue(self.lp.run_exists(FWorker(category="dummy_category")))
+        self.assertFalse(self.lp.run_exists(FWorker(category="other category")))
+        self.assertFalse(self.lp.run_exists(FWorker(category="__none__")))
+        self.assertTrue(self.lp.run_exists(FWorker()))  # can run any category
+        self.assertTrue(self.lp.run_exists(FWorker(category=["dummy_category",
+                                                             "other category"])))
+
+    def test_category_pt2(self):
+        task1 = ScriptTask.from_str('echo "Task 1"')
+        task2 = ScriptTask.from_str('echo "Task 2"')
+
+        fw1 = Firework(task1, fw_id=1, name='Task 1')
+        fw2 = Firework(task2, fw_id=2, name='Task 2')
+
+        self.lp.add_wf(Workflow([fw1, fw2]))
+
+        self.assertFalse(self.lp.run_exists(FWorker(category="dummy_category")))
+        self.assertTrue(self.lp.run_exists(FWorker(category="__none__")))
+        self.assertTrue(self.lp.run_exists(FWorker())) # can run any category
+        self.assertFalse(self.lp.run_exists(FWorker(category=["dummy_category",
+                                                             "other category"])))
     def test_delete_fw(self):
         test1 = ScriptTask.from_str("python -c 'print(\"test1\")'", {'store_stdout': True})
         fw = Firework(test1)
@@ -410,17 +467,20 @@ class MongoTests(unittest.TestCase):
         self.assertRaises(ValueError, self.lp.get_fw_by_id, fw.fw_id)
         self.assertRaises(ValueError, self.lp.get_launch_by_id, 1)
 
-
     def test_duplicate_delete_fw(self):
         test1 = ScriptTask.from_str("python -c 'print(\"test1\")'", {'store_stdout': True})
         fw = Firework(test1, {"_dupefinder": DupeFinderExact()})
         self.lp.add_wf(fw)
         self.lp.add_wf(fw)
         launch_rocket(self.lp, self.fworker)
+        # TODO: if test keeps failing on Travis, add an explicit check of nlaunches>0 in the database here
+        # this will ensure the first Rocket is actually in the DB
         launch_rocket(self.lp, self.fworker)
 
-        self.lp.delete_wf(2)
-        self.assertRaises(ValueError, self.lp.get_fw_by_id, 2)
+        run_id = self.lp.get_launch_by_id(1).fw_id
+        del_id = 1 if run_id == 2 else 2
+        self.lp.delete_wf(del_id)
+        self.assertRaises(ValueError, self.lp.get_fw_by_id, del_id)
         self.assertEqual(self.lp.get_launch_by_id(1).action.stored_data[
             'stdout'], 'test1\n')
 
@@ -430,8 +490,18 @@ class MongoTests(unittest.TestCase):
         self.lp.add_wf(fw)
         self.lp.add_wf(fw)
         launch_rocket(self.lp, self.fworker)
+        # TODO: if test keeps failing on Travis, add an explicit check of nlaunches>0 in the database here
+        # this will ensure the first Rocket is actually in the DB
         launch_rocket(self.lp, self.fworker)
 
+        time.sleep(1)
+
+        if self.lp.launches.count() > 1:
+            print("TOO MANY LAUNCHES FOUND!")
+            print("--------")
+            for d in self.lp.launches.find():
+                print(d)
+            print("--------")
         self.assertEqual(self.lp.launches.count(), 1)
 
     def test_append_wf(self):
@@ -478,7 +548,6 @@ class MongoTests(unittest.TestCase):
 
         self.assertEqual(self.lp.get_fw_by_id(2).spec['dummy2'], [True, True])
 
-
     def test_force_lock_removal(self):
         test1 = ScriptTask.from_str("python -c 'print(\"test1\")'", {'store_stdout': True})
         fw = Firework(test1, {"_dupefinder": DupeFinderExact()}, fw_id=1)
