@@ -1,13 +1,12 @@
-# coding: utf-8
-
-from __future__ import unicode_literals
+""" This module includes tasks to integrate scripts and python functions """
 
 import shlex
 import subprocess
 import sys
-
-from fireworks.core.firework import FireTaskBase, FWAction
+from fireworks.core.firework import FiretaskBase, FWAction
 from six.moves import builtins
+if sys.version_info[0] > 2:
+    basestring = str
 
 __author__ = 'Anubhav Jain'
 __copyright__ = 'Copyright 2013, The Materials Project'
@@ -16,15 +15,14 @@ __maintainer__ = 'Anubhav Jain'
 __email__ = 'ajain@lbl.gov'
 __date__ = 'Feb 18, 2013'
 
-# TODO: add maximum length of 10,000 chars for stored fields
 
-
-class ScriptTask(FireTaskBase):
-    required_params = ["script"]
+class ScriptTask(FiretaskBase):
+    """ Runs a user-defined script """
+    required_params = ['script']
     _fw_name = 'ScriptTask'
 
     def run_task(self, fw_spec):
-        if self.get("use_global_spec"):
+        if self.get('use_global_spec'):
             self._load_params(fw_spec)
         else:
             self._load_params(self)
@@ -54,7 +52,7 @@ class ScriptTask(FireTaskBase):
                 (stdout, stderr) = p.communicate()
             returncodes.append(p.returncode)
 
-            #Stop execution if any script command fails.
+            # stop execution if any script command fails
             if p.returncode != 0:
                 break
 
@@ -93,13 +91,12 @@ class ScriptTask(FireTaskBase):
 
     def _load_params(self, d):
         if d.get('stdin_file') and d.get('stdin_key'):
-            raise ValueError("ScriptTask cannot process both a key and file as the standard in!")
+            raise ValueError('ScriptTask cannot process both a key and file as the standard in!')
 
         self.use_shell = d.get('use_shell', True)
 
         m_script = d['script']
-        if (sys.version_info < (3, 0) and isinstance(m_script, unicode)) or\
-                isinstance(m_script, str):
+        if isinstance(m_script, basestring):
             m_script = [m_script]
 
         if not self.use_shell:
@@ -118,7 +115,7 @@ class ScriptTask(FireTaskBase):
         self.fizzle_bad_rc = d.get('fizzle_bad_rc', not self.defuse_bad_rc)
 
         if self.defuse_bad_rc and self.fizzle_bad_rc:
-            raise ValueError("ScriptTask cannot both FIZZLE and DEFUSE a bad returncode!")
+            raise ValueError('ScriptTask cannot both FIZZLE and DEFUSE a bad returncode!')
 
     @classmethod
     def from_str(cls, shell_cmd, parameters=None):
@@ -128,50 +125,88 @@ class ScriptTask(FireTaskBase):
         return cls(parameters)
 
 
-class PyTask(FireTaskBase):
-    _fw_name = 'PyTask'
-
-    __doc__ = """
+class PyTask(FiretaskBase):
+    """
     Runs any python function! Extremely powerful, which allows you to
-    essentially run any accessible method on the system.
+    essentially run any accessible method on the system. The optional inputs
+    and outputs lists may contain spec keys to add to args list and to make
+    the function output available in the curent and in children fireworks.
 
-    Args:
-        func (str): Fully qualified python method. E.g., json.dump, or shutil
+    Required parameters:
+        - func (str): Fully qualified python method. E.g., json.dump, or shutil
             .copy, or some other function that is not part of the standard
             library!
-        args (list): List of args. Default is empty.
-        kwargs (dict): Dictionary of keyword args. Default is empty.
-        auto_kwargs (bool): If True, all other params not starting with "_" are supplied as keyword args
-        stored_data_varname (str): Whether to store the output in
-            FWAction. If
-            this is a string that does not evaluate to False, the output of
-            the function will be stored as
-            FWAction(stored_data={stored_data_varname: output}). The name is
-            deliberately long to avoid potential name conflicts.
-    """
 
-    required_params = ["func"]
-    optional_params = ["args", "kwargs",  "auto_kwargs", "stored_data_varname"]
+    Optional parameters:
+        - args (list): List of args. Default is empty.
+        - kwargs (dict): Dictionary of keyword args. Default is empty.
+        - auto_kwargs (bool): If True, all other params not starting with '_'
+          are supplied as keyword args
+        - stored_data_varname (str): Whether to store the output in FWAction.
+          If this is a string that does not evaluate to False, the output of
+          the function will be stored as
+          FWAction(stored_data={stored_data_varname: output}). The name is
+          deliberately long to avoid potential name conflicts.
+        - inputs ([str]): a list of keys in spec which will be used as inputs;
+          the generated arguments list will be appended to args
+        - outputs ([str]): a list of spec keys that will be used to pass
+          the function's outputs to child fireworks
+        - chunk_number (int): a serial number of the Firetask within a
+          group of Firetasks generated by a ForeachTask
+    """
+    _fw_name = 'PyTask'
+    required_params = ['func']
+    other_params = ['args', 'kwargs', 'auto_kwargs', 'stored_data_varname',
+                    'inputs', 'outputs', 'chunk_number']
+    # note that we are not using "optional_params" because we do not want to do
+    # strict parameter checking in FireTaskBase due to "auto_kwargs" option
 
     def run_task(self, fw_spec):
-        toks = self["func"].rsplit(".", 1)
+        toks = self['func'].rsplit('.', 1)
         if len(toks) == 2:
             modname, funcname = toks
             mod = __import__(modname, globals(), locals(), [str(funcname)], 0)
             func = getattr(mod, funcname)
         else:
-            #Handle built in functions.
+            # Handle built in functions.
             func = getattr(builtins, toks[0])
 
-        args = self.get("args", [])
-        if self.get("auto_kwargs"):
+        args = list(self.get('args', []))  # defensive copy
+
+        inputs = self.get('inputs', [])
+        assert isinstance(inputs, list)
+        for item in inputs:
+            args.append(fw_spec[item])
+
+        if self.get('auto_kwargs'):
             kwargs = {k: v for k, v in self.items()
-                  if not (k.startswith("_") or k in self.required_params or k in self.optional_params)}
+                      if not (k.startswith('_')
+                              or k in self.required_params
+                              or k in self.other_params)}
         else:
-            kwargs = self.get("kwargs", {})
+            kwargs = self.get('kwargs', {})
 
         output = func(*args, **kwargs)
-        if isinstance(output,FWAction):
+
+        if isinstance(output, FWAction):
             return output
-        elif self.get("stored_data_varname"):
-            return FWAction(stored_data={self["stored_data_varname"]: output})
\ No newline at end of file
+        actions = {}
+        outputs = self.get('outputs', [])
+        assert isinstance(outputs, list)
+        if len(outputs) == 1:
+            if self.get('chunk_number') is None:
+                actions['update_spec'] = {outputs[0]: output}
+            else:
+                if isinstance(output, (list, tuple, set)):
+                    mod_spec = [{'_push': {outputs[0]: i}} for i in output]
+                else:
+                    mod_spec = [{'_push': {outputs[0]: output}}]
+                actions['mod_spec'] = mod_spec
+        elif len(outputs) > 1:
+            assert isinstance(output, (list, tuple, set))
+            assert len(output) == len(outputs)
+            actions['update_spec'] = dict(zip(outputs, output))
+        if self.get('stored_data_varname'):
+            actions['stored_data'] = {self['stored_data_varname']: output}
+        if len(actions) > 0:
+            return FWAction(**actions)
