@@ -1,10 +1,17 @@
+# coding: utf-8
+
+from __future__ import unicode_literals
+
 """
-This module contains the TemplateWriterTask, which writes files based on a template file and a Context using Jinja2's templating engine.
+This module contains the TemplateWriterTask, which writes files based on a template file and a
+Context using Jinja2's templating engine.
 """
 
 import os
-from jinja2 import Template
-from fireworks.core.firework import FireTaskBase
+
+from jinja2 import Environment, FileSystemLoader
+
+from fireworks.core.firework import FiretaskBase
 from fireworks.fw_config import TEMPLATE_DIR
 
 __author__ = 'Anubhav Jain'
@@ -15,7 +22,7 @@ __email__ = 'ajain@lbl.gov'
 __date__ = 'Aug 08, 2013'
 
 
-class TemplateWriterTask(FireTaskBase):
+class TemplateWriterTask(FiretaskBase):
     """
     Task to write templated files via Jinja2 library
     Required parameters:
@@ -35,7 +42,8 @@ class TemplateWriterTask(FireTaskBase):
             self._load_params(self)
 
         with open(self.template_file) as f:
-            t = Template(f.read())
+            t = Environment(loader=FileSystemLoader(self.template_dir),
+                            autoescape=True).from_string(f.read())
             output = t.render(self.context)
 
             write_mode = 'w+' if self.append_file else 'w'
@@ -58,4 +66,5 @@ class TemplateWriterTask(FireTaskBase):
 
         self.template_file = os.path.join(self.template_dir, d['template_file'])
         if not os.path.exists(self.template_file):
-            raise ValueError("TemplateWriterTask could not find a template file at: {}".format(self.template_file))
\ No newline at end of file
+            raise ValueError("TemplateWriterTask could not find a template file at: {}".format(
+                self.template_file))
