@@ -3,6 +3,8 @@
 from __future__ import unicode_literals
 import copy
 
+import sys
+
 """
 This module implements a CommonAdaptor that supports standard PBS and SGE
 queues.
@@ -37,10 +39,11 @@ class CommonAdapter(QueueAdapterBase):
         "Cobalt": {"submit_cmd": "qsub", "status_cmd": "qstat"},
         "SLURM": {"submit_cmd": "sbatch", "status_cmd": "squeue"},
         "LoadLeveler": {"submit_cmd": "llsubmit", "status_cmd": "llq"},
-        "LoadSharingFacility": {"submit_cmd": "bsub", "status_cmd": "bjobs"}
+        "LoadSharingFacility": {"submit_cmd": "bsub", "status_cmd": "bjobs"},
+        "MOAB": {"submit_cmd": "msub", "status_cmd": "showq"}
     }
 
-    def __init__(self, q_type, q_name=None, template_file=None, **kwargs):
+    def __init__(self, q_type, q_name=None, template_file=None, timeout=None, **kwargs):
         """
         :param q_type: The type of queue. Right now it should be either PBS,
                        SGE, SLURM, Cobalt or LoadLeveler.
@@ -49,6 +52,8 @@ class CommonAdapter(QueueAdapterBase):
                               None (the default) to use Fireworks' built-in
                               templates for PBS and SGE, which should work
                               on most queues.
+        :param timeout: The amount of seconds to wait before raising an error when
+                        checking the number of jobs in the queue. Default 5 seconds.
         :param **kwargs: Series of keyword args for queue parameters.
         """
         if q_type not in CommonAdapter.default_q_commands:
@@ -60,6 +65,7 @@ class CommonAdapter(QueueAdapterBase):
         self.template_file = os.path.abspath(template_file) if template_file is not None else \
             CommonAdapter._get_default_template_file(q_type)
         self.q_name = q_name or q_type
+        self.timeout = timeout or 5
         self.update(dict(kwargs))
 
         self.q_commands = copy.deepcopy(CommonAdapter.default_q_commands)
@@ -68,17 +74,17 @@ class CommonAdapter(QueueAdapterBase):
 
     def _parse_jobid(self, output_str):
         if self.q_type == "SLURM":
-            for l in output_str.split("\n"):
-                if l.startswith("Submitted batch job"):
-                    return int(l.split()[-1])
-        if self.q_type == "LoadLeveler":
+            # The line can contain more text after the id.
+            # Match after the standard "Submitted batch job" string
+            re_string = r"Submitted batch job\s+(\d+)"
+        elif self.q_type == "LoadLeveler":
             # Load Leveler: "llsubmit: The job "abc.123" has been submitted"
             re_string = r"The job \"(.*?)\" has been submitted"
         elif self.q_type == "Cobalt":
             # 99% of the time you just get:
             # Cobalt: "199768"
             # but there's a version that also includes project and queue
-            # information on preceeding lines and both of those  might 
+            # information on preceeding lines and both of those  might
             # contain a number in any position.
             re_string = r"(\b\d+\b)"
         else:
@@ -100,16 +106,22 @@ class CommonAdapter(QueueAdapterBase):
             # -h: no header line
             # -o: reduce output to user only (shorter string to parse)
             status_cmd.extend(['-o "%u"', '-u', username, '-h'])
-            if 'queue' in self and self['queue']:
+            if self.get('queue'):
                 status_cmd.extend(['-p', self['queue']])
         elif self.q_type == "LoadSharingFacility":
-            #use no header and the wide format so that there is one line per job, and display only running and pending jobs
-            status_cmd.extend(['-p','-r','-o', 'jobID user queue', '-noheader', '-u', username])
+            # use no header and the wide format so that there is one line per job, and display only running and
+            # pending jobs
+            status_cmd.extend(['-p', '-r', '-o', 'jobID user queue', '-noheader', '-u', username])
         elif self.q_type == "Cobalt":
-            header="JobId:User:Queue:Jobname:Nodes:Procs:Mode:WallTime:State:RunTime:Project:Location"
+            header = "JobId:User:Queue:Jobname:Nodes:Procs:Mode:WallTime:State:RunTime:Project:Location"
             status_cmd.extend(['--header', header, '-u', username])
         elif self.q_type == 'SGE':
-            status_cmd.extend(['-q', self['queue'], '-u', username])
+            status_cmd.extend(['-u', username])
+            if self.get('queue'):
+                status_cmd.extend(['-q', self['queue']])
+        elif self.q_type == "MOAB":
+            status_cmd.extend(['-w', "user={}".format(username)])
+            # no queue restriction command known for QUEST supercomputer, i.e., -p option doesn't work
         else:
             status_cmd.extend(['-u', username])
 
@@ -124,7 +136,7 @@ class CommonAdapter(QueueAdapterBase):
 
         if self.q_type == 'SLURM':
             # subtract one due to trailing '\n' and split behavior
-            return len(output_str.split('\n'))-1
+            return len(output_str.split('\n')) - 1
 
         if self.q_type == "LoadLeveler":
             if 'There is currently no job status to report' in output_str:
@@ -133,22 +145,31 @@ class CommonAdapter(QueueAdapterBase):
                 # last line is: "1 job step(s) in query, 0 waiting, ..."
                 return int(output_str.split('\n')[-2].split()[0])
         if self.q_type == "LoadSharingFacility":
-            #Just count the number of lines
-            return len(output_str.split('\n'))
+            # Count the number of lines which pertain to the queue
+            cnt = 0
+            for line in output_str.split('\n'):
+                if line.endswith(self['queue']):
+                    cnt += 1
+            return cnt
         if self.q_type == "SGE":
             # want only lines that include username;
             # this will exclude e.g. header lines
             return len([l for l in output_str.split('\n') if username in l])
 
+        if self.q_type == "MOAB":
+            # want only lines that include username;
+            # this will exclude e.g. header lines
+            return len([l for l in output_str.split('\n') if username in l])
+
         count = 0
         for l in output_str.split('\n'):
             if l.lower().startswith("job"):
                 if self.q_type == "Cobalt":
                     # Cobalt capitalzes headers
-                    l=l.lower()
+                    l = l.lower()
                 header = l.split()
                 if self.q_type == "PBS":
-                    #PBS has a ridiculous two word "Job ID" in header
+                    # PBS has a ridiculous two word "Job ID" in header
                     state_index = header.index("S") - 1
                     queue_index = header.index("Queue") - 1
                 else:
@@ -182,22 +203,22 @@ class CommonAdapter(QueueAdapterBase):
         try:
             if self.q_type == "Cobalt":
                 # Cobalt requires scripts to be executable
-                os.chmod(script_file,stat.S_IRWXU|stat.S_IRGRP|stat.S_IXGRP)
+                os.chmod(script_file, stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP)
             cmd = [submit_cmd, script_file]
-            #For most of the queues handled by common_adapter, it's best to simply submit the file name
-            #as an argument.  LoadSharingFacility doesn't handle the header section (queue name, nodes, etc)
-            #when taking file arguments, so the file needs to be passed as stdin to make it work correctly.
+            # For most of the queues handled by common_adapter, it's best to simply submit the file name
+            # as an argument.  LoadSharingFacility doesn't handle the header section (queue name, nodes, etc)
+            # when taking file arguments, so the file needs to be passed as stdin to make it work correctly.
             if self.q_type == 'LoadSharingFacility':
                 with open(script_file, 'r') as inputFile:
-                    p = subprocess.Popen([submit_cmd],stdin=inputFile,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
+                    p = subprocess.Popen([submit_cmd], stdin=inputFile, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             else:
                 p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             p.wait()
 
-            # grab the returncode. PBS returns 0 if the job was successful
+            # retrieve the returncode. PBS returns 0 if the job was successful
             if p.returncode == 0:
                 try:
-                    job_id = self._parse_jobid(p.stdout.read())
+                    job_id = self._parse_jobid(p.stdout.read().decode())
                     queue_logger.info(
                         'Job submission was successful and job_id is {}'.format(
                             job_id))
@@ -236,7 +257,7 @@ class CommonAdapter(QueueAdapterBase):
 
         # run qstat
         qstat = Command(self._get_status_cmd(username))
-        p = qstat.run(timeout=5)
+        p = qstat.run(timeout=self.timeout)
 
         # parse the result
         if p[0] == 0:
@@ -265,6 +286,7 @@ class CommonAdapter(QueueAdapterBase):
             d["_fw_q_name"] = self.q_name
         if self.template_file != CommonAdapter._get_default_template_file(self.q_type):
             d["_fw_template_file"] = self.template_file
+        d["_fw_timeout"] = self.timeout
         return d
 
     @classmethod
@@ -273,4 +295,5 @@ class CommonAdapter(QueueAdapterBase):
             q_type=m_dict["_fw_q_type"],
             q_name=m_dict.get("_fw_q_name"),
             template_file=m_dict.get("_fw_template_file"),
+            timeout=m_dict.get("_fw_timeout"),
             **{k: v for k, v in m_dict.items() if not k.startswith("_fw")})
