@@ -13,6 +13,7 @@ __maintainer__ = 'Anubhav Jain'
 __email__ = 'ajain@lbl.gov'
 __date__ = 'Nov 21, 2013'
 
+
 class PBSAdapterNEWT(QueueAdapterBase):
     """
     A special PBS adapter that works via the NEWT interface (https://newt.nersc.gov)
@@ -29,7 +30,8 @@ class PBSAdapterNEWT(QueueAdapterBase):
     def submit_to_queue(self, script_file):
         self._init_auth_session()
         jobfile = os.path.join(os.getcwd(), script_file)
-        r = PBSAdapterNEWT._session.post("https://newt.nersc.gov/newt/queue/{}/".format(self.resource), {"jobfile": jobfile})
+        r = PBSAdapterNEWT._session.post("https://newt.nersc.gov/newt/queue/{}/".format(self.resource),
+                                         {"jobfile": jobfile})
         return int(r.json()['jobid'].split('.')[0])
 
     def get_njobs_in_queue(self, username=None):
@@ -39,7 +41,8 @@ class PBSAdapterNEWT(QueueAdapterBase):
         r = Session().get("https://newt.nersc.gov/newt/queue/{}/?user={}".format(self.resource, username))
         return len(r.json())
 
-    def _init_auth_session(self, max_pw_requests=3):
+    @staticmethod
+    def _init_auth_session(max_pw_requests=3):
         """
         Initialize the _session class var with an authorized session. Asks for a /
         password in new sessions, skips PW check for previously authenticated sessions
@@ -57,7 +60,8 @@ class PBSAdapterNEWT(QueueAdapterBase):
         pw_iterations = 0
         while pw_iterations < max_pw_requests:
             password = getpass.getpass()
-            r = PBSAdapterNEWT._session.post("https://newt.nersc.gov/newt/auth", {"username": username, "password": password})
+            r = PBSAdapterNEWT._session.post("https://newt.nersc.gov/newt/auth",
+                                             {"username": username, "password": password})
             if r.json()['auth'] and r.json()['username'] == username:
                 return
             pw_iterations += 1
