@@ -1,30 +1,32 @@
 # coding: utf-8
 
 from __future__ import unicode_literals
-from monty.json import MontyDecoder, MSONable
 
-__doc__ = """
+"""
 This module aids in serializing and deserializing objects.
 
-To serialize a FW object, refer to the documentation for the FWSerializable class. To de-serialize an object,
-refer to the documentation for the FWSerializable class and load_object() method.
+To serialize a FW object, refer to the documentation for the FWSerializable class. To de-serialize
+an object, refer to the documentation for the FWSerializable class and load_object() method.
     - you can de-serialize explicitly, e.g. FWObject.from_dict() to enforce a FWObject instance as the result
     - you can de-serialize implicitly, e.g. load_object() to search for the correct Class dynamically
 
-The implicit method is especially useful if you don't know in advance which subclass of a base class your \
-serialization might point to. e.g. if you require some type of Quadrilateral, a serialization from your \
-collaborator might point to a Square, Rhombus, or Rectangle, and you might not know which one in advance...
+The implicit method is especially useful if you don't know in advance which subclass of a base class
+your serialization might point to. e.g. if you require some type of Quadrilateral, a serialization
+from your collaborator might point to a Square, Rhombus, or Rectangle, and you might not know which
+one in advance...
 
 Some advantages:
-    - Robust with regard to code refactorings even in implicit loading given certain reasonable guidelines on fw_name.
-    - Simple to allow a superclass to define all the serializations for its subclasses, removing code repetition
-    (in particular, note that from_dict is a class method rather than a static method, allowing use of self)
+    - Robust with regard to code refactorings even in implicit loading given certain reasonable
+        guidelines on fw_name.
+    - Simple to allow a superclass to define all the serializations for its subclasses, removing
+        code repetition(in particular, note that from_dict is a class method rather than a static
+        method, allowing use of self)
     - Decorators aid in some of the routine parts of the serialization, such as adding the _fw_name key
     - Both JSON and YAML file import/export are naturally and concisely supported within the framework.
     - Auto-detect and proper loading of JSON and YAML files
     - Proper JSON handling of datetime (both encoding and decoding) and UTF-8 strings
-    - In some cases, objects can be serialized/deserialized extremely concisely, by use of only their fw_name (if no
-    parameters are needed to describe the object)
+    - In some cases, objects can be serialized/deserialized extremely concisely, by use of only
+        their fw_name (if no parameters are needed to describe the object)
 
 """
 
@@ -36,15 +38,9 @@ import importlib
 import datetime
 import abc
 import sys
-
-import yaml
-# Use CLoader for faster performance where possible.
-try:
-    from yaml import CLoader as Loader, CSafeDumper as Dumper
-except ImportError:
-    from yaml import Loader as Loader, SafeDumper as Dumper
 import six
-
+import ruamel.yaml as yaml
+from monty.json import MontyDecoder, MSONable
 from fireworks.fw_config import FW_NAME_UPDATES, YAML_STYLE, USER_PACKAGES, DECODE_MONTY, ENCODE_MONTY
 
 __author__ = 'Anubhav Jain'
@@ -54,13 +50,23 @@ __maintainer__ = 'Anubhav Jain'
 __email__ = 'ajain@lbl.gov'
 __date__ = 'Dec 13, 2012'
 
+# TODO: consider *somehow* switching FireWorks to monty serialization. e.g., numpy serialization is better handled.
+
 SAVED_FW_MODULES = {}
 DATETIME_HANDLER = lambda obj: obj.isoformat() if isinstance(obj, datetime.datetime) else None
+
 if sys.version_info > (3, 0, 0):
     ENCODING_PARAMS = {"encoding": "utf-8"}
 else:
     ENCODING_PARAMS = {}
 
+try:
+    import numpy as np
+
+    NUMPY_INSTALLED = True
+except Exception:
+    NUMPY_INSTALLED = False
+
 
 def recursive_dict(obj, preserve_unicode=True):
     if obj is None:
@@ -73,7 +79,8 @@ def recursive_dict(obj, preserve_unicode=True):
         return recursive_dict(obj.to_dict(), preserve_unicode)
 
     if isinstance(obj, dict):
-        return {recursive_dict(k, preserve_unicode): recursive_dict(v, preserve_unicode) for k, v in obj.items()}
+        return {recursive_dict(k, preserve_unicode): recursive_dict(v, preserve_unicode)
+                for k, v in obj.items()}
 
     if isinstance(obj, (list, tuple)):
         return [recursive_dict(v, preserve_unicode) for v in obj]
@@ -87,6 +94,9 @@ def recursive_dict(obj, preserve_unicode=True):
     if preserve_unicode and isinstance(obj, six.text_type) and obj != obj.encode('ascii', 'ignore'):
         return obj
 
+    if NUMPY_INSTALLED and isinstance(obj, np.ndarray):
+        return [recursive_dict(v, preserve_unicode) for v in obj.tolist()]
+
     return str(obj)
 
 
@@ -114,7 +124,7 @@ def _recursive_load(obj):
         try:
             # convert String to datetime if really datetime
             return reconstitute_dates(obj)
-        except:
+        except Exception:
             # convert unicode to ASCII if not really unicode
             if obj == obj.encode('ascii', 'ignore'):
                 return str(obj)
@@ -200,7 +210,8 @@ class FWSerializable(object):
 
     def as_dict(self):
         # strictly for pseudo-compatibility with MSONable
-        # Note that FWSerializable is not MSONable, it uses _fw_name instead of __class__ and __module__
+        # Note that FWSerializable is not MSONable, it uses _fw_name instead of __class__ and
+        # __module__
         return self.to_dict()
 
     @classmethod
@@ -214,37 +225,46 @@ class FWSerializable(object):
     def to_format(self, f_format='json', **kwargs):
         """
         returns a String representation in the given format
-        :param f_format: the format to output to (default json)
+
+        Args:
+            f_format (str): the format to output to (default json)
         """
         if f_format == 'json':
             return json.dumps(self.to_dict(), default=DATETIME_HANDLER, **kwargs)
         elif f_format == 'yaml':
             # start with the JSON format, and convert to YAML
-            return yaml.dump(self.to_dict(), default_flow_style=YAML_STYLE,
-                             allow_unicode=True, Dumper=Dumper)
+            return yaml.safe_dump(self.to_dict(), default_flow_style=YAML_STYLE,
+                                  allow_unicode=True)
         else:
             raise ValueError('Unsupported format {}'.format(f_format))
 
     @classmethod
     def from_format(cls, f_str, f_format='json'):
         """
-        convert from a String representation to its Object
-        :param f_str: the String representation
-        :param f_format: serialization format of the String (default json)
+        convert from a String representation to its Object.
+
+        Args:
+            f_str (str): the String representation
+            f_format (str): serialization format of the String (default json)
+
+        Returns:
+            FWSerializable
         """
         if f_format == 'json':
             return cls.from_dict(reconstitute_dates(json.loads(f_str)))
         elif f_format == 'yaml':
             return cls.from_dict(reconstitute_dates(
-                yaml.load(f_str, Loader=Loader)))
+                yaml.safe_load(f_str)))
         else:
             raise ValueError('Unsupported format {}'.format(f_format))
 
     def to_file(self, filename, f_format=None, **kwargs):
         """
-        Write a serialization of this object to a file
-        :param filename: filename to write to
-        :param f_format: serialization format, default checks the filename extension
+        Write a serialization of this object to a file.
+
+        Args:
+            filename(str): filename to write to
+            f_format (str): serialization format, default checks the filename extension
         """
         if f_format is None:
             f_format = filename.split('.')[-1]
@@ -254,9 +274,14 @@ class FWSerializable(object):
     @classmethod
     def from_file(cls, filename, f_format=None):
         """
-        Load a serialization of this object from a file
-        :param filename: filename to read
-        :param f_format: serialization format, default checks the filename extension
+        Load a serialization of this object from a file.
+
+        Args:
+            filename (str): filename to read
+            f_format (str): serialization format, default checks the filename extension
+
+        Returns:
+            FWSerializable
         """
         if f_format is None:
             f_format = filename.split('.')[-1]
@@ -293,7 +318,8 @@ def load_object(obj_dict):
     i) If you want to change the fw_name of an object you can set the FW_NAME_UPDATES key
     ii) If you want to put a refactored module in a new place add an entry to USER_PACKAGES
 
-    :param obj_dict: the dict representation of the class
+    Args:
+        obj_dict (dict): the dict representation of the class
     """
 
     # override the name in the obj_dict if there's an entry in FW_NAME_UPDATES
@@ -318,10 +344,10 @@ def load_object(obj_dict):
     # failing that, look for the object within all of USER_PACKAGES
     # this will be slow, but only needed the first time
 
-    found_objects = [] # used to make sure we don't find multiple hits
+    found_objects = []  # used to make sure we don't find multiple hits
     for package in USER_PACKAGES:
         root_module = importlib.import_module(package)
-        for loader, mod_name, is_pkg in pkgutil.walk_packages(
+        for _, mod_name, is_pkg in pkgutil.walk_packages(
                 root_module.__path__, package + '.'):
             try:
                 m_module = importlib.import_module(mod_name)
@@ -340,8 +366,7 @@ def load_object(obj_dict):
         return found_objects[0][0]
     elif len(found_objects) > 0:
         raise ValueError(
-            'load_object() found multiple objects with cls._fw_name {} -- {}'
-            .format(fw_name, found_objects))
+            'load_object() found multiple objects with cls._fw_name {} -- {}'.format(fw_name, found_objects))
 
     raise ValueError('load_object() could not find a class with cls._fw_name {}'.format(fw_name))
 
@@ -351,11 +376,11 @@ def load_object_from_file(filename, f_format=None):
     Implicitly load an object from a file. just a friendly wrapper to
     load_object()
 
-    :param filename: the filename to load an object from
-    :param f_format: the serialization format (default is auto-detect based on
-        filename extension)
+    Args:
+        filename (str): the filename to load an object from
+        f_format (str): the serialization format (default is auto-detect based on
+            filename extension)
     """
-
     m_dict = {}
     if f_format is None:
         f_format = filename.split('.')[-1]
@@ -364,7 +389,7 @@ def load_object_from_file(filename, f_format=None):
         if f_format == 'json':
             m_dict = reconstitute_dates(json.loads(f.read()))
         elif f_format == 'yaml':
-            m_dict = reconstitute_dates(yaml.load(f, Loader=Loader))
+            m_dict = reconstitute_dates(yaml.safe_load(f))
         else:
             raise ValueError('Unknown file format {} cannot be loaded!'.format(f_format))
 
@@ -377,7 +402,7 @@ def _search_module_for_obj(m_module, obj_dict):
     """
     obj_name = obj_dict['_fw_name']
 
-    for name, obj in inspect.getmembers(m_module):
+    for _, obj in inspect.getmembers(m_module):
         # check if the member is a Class matching our description
         if inspect.isclass(obj) and obj.__module__ == m_module.__name__ and \
                 getattr(obj, '_fw_name', get_default_serialization(obj)) == obj_name:
@@ -397,12 +422,11 @@ def reconstitute_dates(obj_dict):
     if isinstance(obj_dict, six.string_types):
         try:
             return datetime.datetime.strptime(obj_dict, "%Y-%m-%dT%H:%M:%S.%f")
-        except:
+        except Exception:
             try:
                 return datetime.datetime.strptime(obj_dict, "%Y-%m-%dT%H:%M:%S")
-            except:
+            except Exception:
                 pass
-
     return obj_dict
 
 
