@@ -2,7 +2,7 @@
 
 import logging
 import datetime
-from multiprocessing.managers import BaseManager, DictProxy
+from multiprocessing.managers import BaseManager
 import string
 import sys
 import os
@@ -10,10 +10,10 @@ import traceback
 import socket
 import multiprocessing
 import errno
+import six
+import contextlib
 
-from fireworks.fw_config import FWData, FW_BLOCK_FORMAT, DS_PASSWORD, \
-    FW_LOGGING_FORMAT
-
+from fireworks.fw_config import FWData, FW_BLOCK_FORMAT, DS_PASSWORD, FW_LOGGING_FORMAT
 
 __author__ = 'Anubhav Jain, Xiaohui Qu'
 __copyright__ = 'Copyright 2012, The Materials Project'
@@ -27,20 +27,19 @@ PREVIOUS_FILE_LOGGERS = []  # contains the name of file loggers that have alread
 DEFAULT_FORMATTER = logging.Formatter(FW_LOGGING_FORMAT)
 
 
-def get_fw_logger(name, l_dir=None, file_levels=('DEBUG', 'ERROR'),
-                  stream_level='DEBUG', formatter=DEFAULT_FORMATTER,
-                  clear_logs=False):
+def get_fw_logger(name, l_dir=None, file_levels=('DEBUG', 'ERROR'), stream_level='DEBUG',
+                  formatter=DEFAULT_FORMATTER, clear_logs=False):
     """
     Convenience method to return a logger.
 
-    :param name: name of the logger that sets the groups, e.g. 'group1.set2'
-    :param l_dir: the directory to put the log file
-    :param file_levels: iterable describing level(s) to log to file(s). default: ('DEBUG', 'ERROR')
-    :param stream_level: level to log to standard output. default: 'DEBUG'
-    :param formatter: logging format. default: FW_LOGGING_FORMATTER
-    :param clear_logs: whether to clear the logger with the same name
+    Args:
+        name: name of the logger that sets the groups, e.g. 'group1.set2'
+        l_dir: the directory to put the log file
+        file_levels: iterable describing level(s) to log to file(s). default: ('DEBUG', 'ERROR')
+        stream_level: level to log to standard output. default: 'DEBUG'
+        formatter: logging format. default: FW_LOGGING_FORMATTER
+        clear_logs: whether to clear the logger with the same name
     """
-
     logger = logging.getLogger(name)
     logger.setLevel(logging.DEBUG)  # anything debug and above passes through to the handler level
 
@@ -70,9 +69,10 @@ def get_fw_logger(name, l_dir=None, file_levels=('DEBUG', 'ERROR'),
 
 def log_multi(m_logger, msg, log_lvl='info'):
     """
-    :param m_logger: (logger) The logger object
-    :param msg: (str) a String to log
-    :param log_lvl: (str) The level to log at
+    Args:
+        m_logger (logger): The logger object
+        msg (str): a String to log
+        log_lvl (str): The level to log at
     """
     _log_fnc = getattr(m_logger, log_lvl.lower())
     if FWData().MULTIPROCESSING:
@@ -88,13 +88,13 @@ def log_fancy(m_logger, msgs, log_lvl='info', add_traceback=False):
     which enhances readability of log lines meant to be read
     as a unit.
 
-    :param m_logger: (logger) The logger object
-    :param log_lvl: (str) The level to log at
-    :param msgs: ([str]) a String or iterable of Strings
-    :param add_traceback: (bool) add traceback text, useful when logging exceptions (default False)
+    Args:
+        m_logger (logger): The logger object
+        log_lvl (str): The level to log at
+        msgs ([str]): a String or iterable of Strings
+        add_traceback (bool): add traceback text, useful when logging exceptions (default False)
     """
-
-    if isinstance(msgs, basestring):
+    if isinstance(msgs, six.string_types):
         msgs = [msgs]
 
     _log_fnc = getattr(m_logger, log_lvl.lower())
@@ -110,26 +110,29 @@ def log_exception(m_logger, msgs):
     """
     A shortcut wrapper around log_fancy for exceptions
 
-    :param m_logger: (logger) The logger object
-    :param msgs: ([str]) String or iterable of Strings, will be joined by newlines
+    Args:
+        m_logger (logger): The logger object
+        msgs ([str]): String or iterable of Strings, will be joined by newlines
     """
     return log_fancy(m_logger, msgs, 'error', add_traceback=True)
 
 
 def create_datestamp_dir(root_dir, l_logger, prefix='block_'):
     """
-    Internal method to create a new block or launcher directory. \
+    Internal method to create a new block or launcher directory.
     The dir name is based on the time and the FW_BLOCK_FORMAT
 
-    :param root_dir: directory to create the new dir in
-    :param l_logger: the logger to use
-    :param prefix: the prefix for the new dir, default="block_"
+    Args:
+        root_dir: directory to create the new dir in
+        l_logger: the logger to use
+        prefix: the prefix for the new dir, default="block_"
     """
 
     def get_path():
         time_now = datetime.datetime.utcnow().strftime(FW_BLOCK_FORMAT)
         block_path = prefix + time_now
         return os.path.join(root_dir, block_path)
+
     ctn = 0
     max_try = 10
     full_path = None
@@ -139,7 +142,7 @@ def create_datestamp_dir(root_dir, l_logger, prefix='block_'):
             full_path = None
             import time
             import random
-            time.sleep(random.random()/3+0.1)
+            time.sleep(random.random() / 3 + 0.1)
             continue
         else:
             try:
@@ -158,12 +161,13 @@ def create_datestamp_dir(root_dir, l_logger, prefix='block_'):
 
 _g_ip, _g_host = None, None
 
+
 def get_my_ip():
     global _g_ip
     if _g_ip is None:
         try:
             _g_ip = socket.gethostbyname(socket.gethostname())
-        except:
+        except Exception:
             _g_ip = '127.0.0.1'
     return _g_ip
 
@@ -191,8 +195,11 @@ class DataServer(BaseManager):
     @classmethod
     def setup(cls, launchpad):
         """
-        :param launchpad: (LaunchPad) object
-        :return:
+        Args:
+            launchpad (LaunchPad)
+
+        Returns:
+            DataServer
         """
         DataServer.register('LaunchPad', callable=lambda: launchpad)
         m = DataServer(address=('127.0.0.1', 0), authkey=DS_PASSWORD)  # random port
@@ -207,6 +214,7 @@ class NestedClassGetter(object):
     and the name of the nested class as the second argument,
     returns an instance of the nested class.
     """
+
     def __call__(self, containing_class, class_name):
         nested_class = getattr(containing_class, class_name)
         # return an instance of a nested_class. Some more intelligence could be
@@ -221,4 +229,98 @@ def explicit_serialize(o):
         import __main__
         module_name = os.path.splitext(os.path.basename(__main__.__file__))[0]
     o._fw_name = '{{%s.%s}}' % (module_name, o.__name__)
-    return o
\ No newline at end of file
+    return o
+
+
+def plot_wf(wf, depth_factor=1.0, breadth_factor=2.0, labels_on=True, numerical_label=False,
+            text_loc_factor=1.0, save_as=None, style='rD--', markersize=10, markerfacecolor='blue',
+            fontsize=12, ):
+    """
+    Generate a visual representation of the workflow. Useful for checking whether the firework
+    connections are in order before launching the workflow.
+
+    Args:
+        wf (Workflow): workflow object.
+        depth_factor (float): adjust this to stretch the plot in y direction.
+        breadth_factor (float): adjust this to stretch the plot in x direction.
+        labels_on (bool): whether to label the nodes or not. The default is to lable the nodes
+            using the firework names.
+        numerical_label (bool): set this to label the nodes using the firework ids.
+        text_loc_factor (float): adjust the label location.
+        save_as (str): save the figure to the given name.
+        style (str): marker style.
+        markersize (int): marker size.
+        markerfacecolor (str): marker face color.
+        fontsize (int): font size for the node label.
+    """
+    try:
+        import matplotlib.pyplot as plt
+    except ImportError:
+        print("Install matplotlib. Exiting.")
+        sys.exit()
+
+    keys = sorted(wf.links.keys(), reverse=True)
+    n_root_nodes = len(wf.root_fw_ids)
+
+    # set (x,y) coordinates for each node in the workflow links
+    points_map = {}
+
+    # root nodes
+    for i, k in enumerate(wf.root_fw_ids):
+        points_map.update({k: ((-0.5 * n_root_nodes + i) * breadth_factor, (keys[0] + 1) * depth_factor)})
+
+    # the rest
+    for k in keys:
+        for i, j in enumerate(wf.links[k]):
+            if not points_map.get(j, None):
+                points_map[j] = (
+                    (i - len(wf.links[k]) / 2.0) * breadth_factor, k * depth_factor)
+
+    # connect the dots
+    for k in keys:
+        for i in wf.links[k]:
+            plt.plot([points_map[k][0], points_map[i][0]], [points_map[k][1], points_map[i][1]],
+                     style, markersize=markersize, markerfacecolor=markerfacecolor)
+            if labels_on:
+                label1 = wf.id_fw[k].name
+                label2 = wf.id_fw[i].name
+                if numerical_label:
+                    label1 = str(k)
+                    label2 = str(i)
+                plt.text(points_map[k][0] * text_loc_factor, points_map[k][1] * text_loc_factor,
+                         label1, fontsize=fontsize)
+                plt.text(points_map[i][0] * text_loc_factor, points_map[i][1] * text_loc_factor,
+                         label2, fontsize=fontsize)
+
+    plt.axis('scaled')
+    plt.axis('off')
+
+    if save_as:
+        plt.savefig(save_as)
+
+
+@contextlib.contextmanager
+def redirect_local():
+    """
+    temporarily redirect stdout or stderr to fws.error and fws.out
+
+    """
+
+    try:
+        old_err = os.dup(sys.stderr.fileno())
+        old_out = os.dup(sys.stdout.fileno())
+
+        new_err = open('FW_job.error', 'w')
+        new_out = open('FW_job.out', 'w')
+
+        os.dup2(new_err.fileno(), sys.stderr.fileno())
+        os.dup2(new_out.fileno(), sys.stdout.fileno())
+        yield
+
+    finally:
+
+        os.dup2(old_err, sys.stderr.fileno())
+        os.dup2(old_out, sys.stdout.fileno())
+
+        new_err.close()
+        new_out.close()
