@@ -1,9 +1,10 @@
-"""
-TODO: add docs
-"""
+# coding: utf-8
+
+#from __future__ import unicode_literals
+
 import sys
 from fireworks.user_objects.firetasks.unittest_tasks import TestSerializer, ExportTestSerializer
-from fireworks.utilities.fw_serializers import load_object, FWSerializable
+from fireworks.utilities.fw_serializers import load_object, FWSerializable, recursive_dict
 from fireworks.utilities.fw_utilities import explicit_serialize
 
 
@@ -128,6 +129,19 @@ class SerializationTest(unittest.TestCase):
         self.assertEqual(load_object({"a": {"p1": {"p2": 3}}, "_fw_name": "TestSerializer Export Name"}), self.obj_4,
                          'Implicit import fails!')
 
+    def test_as_dict(self):
+        self.assertEqual(self.obj_1.as_dict(), self.obj_1.to_dict())
+
+    def test_numpy_array(self):
+        try:
+            import numpy as np
+        except Exception:
+            raise unittest.SkipTest("Skipping numpy serialization testing...")
+
+        x = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+        x = recursive_dict(x)
+        self.assertEqual(x, [[1, 2, 3], [4, 5, 6], [7, 8, 9]])
+
 
 class ExplicitSerializationTest(unittest.TestCase):
     def setUp(self):
