@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-from fireworks.core.firework import FireTaskBase, FWAction
+from fireworks.core.firework import FiretaskBase, FWAction
 
 __author__ = 'Anubhav Jain'
 __copyright__ = 'Copyright 2013, The Materials Project'
@@ -10,7 +10,7 @@ __email__ = 'ajain@lbl.gov'
 __date__ = 'Feb 25, 2013'
 
 
-class AddModifyTask(FireTaskBase):
+class AddModifyTask(FiretaskBase):
 
     _fw_name = "Add and Modify Task"
 
